@@ -6,7 +6,11 @@ namespace py = pybind11;
  * modules aren't preserved over a finalize/initialize.
  */
 
-PYBIND11_MODULE(external_module, m, py::mod_gil_not_used()) {
+PYBIND11_MODULE(external_module,
+                m,
+                py::mod_gil_not_used(),
+                py::multiple_interpreters::per_interpreter_gil()) {
+
     class A {
     public:
         explicit A(int value) : v{value} {};
